%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) María Pérez Ortiz (i82perom at uco dot es)
% 
% This code implements the kernelised versions of the Proportional Odds
% Model presented in the paper: Kernelising the Proportional Odds Model
% through Kernel Learning techniques
% 
% The code has been tested with Ubuntu 12.04 x86_64 and Matlab R2009a
% 
% If you use this code, please cite the associated paper
% Code updates and citing information:
% http://www.uco.es/grupos/ayrna/neucom-kpom
% 
% AYRNA Research group's website:
% http://www.uco.es/ayrna 
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 3
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
% Licence available at: http://www.gnu.org/licenses/gpl-3.0.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [model_information, mae_error] = run_kPOM(train, test, param, options)
% Run the kernelised POM algorithm proposed in ... 
% Input variables
% *train: struct composed of: 
%  train.patterns: Nxd (number of patterns x dimensionality) Train patterns
%  train.targets: Nx1, targets associated to each train pattern
% *test: struct with the same structure than train
% *param: struct variable composed of the different parameters to run the
%  algorithm: 
%  -param.kernelSigma: kernel width for the Gaussian kernel function 
%   (if optimising a combination of p kernels, p values have to be provided)
%  -param.kernelWeight: Kernel weight for each kernel matrix 
%   (only if the combination of kernels is on)         
%  -param.EFSdimensions: Fixed number of dimensions for the EFS 
%   (only if the number of dimensions is not to be optimised)
% *options: struct variable to run different proposals:
%  -options.kernelized: Boolean variable to run the kernelized or simple
%  version of the POM algorithm
%  -options.kernelLearning: Boolean variable to optimise a combination of
%   kernels (1-optimise, 0-use only one kernel using param.kernelSigma)
%  -options.ordinal: Boolean variable to use the ordinal weight matrix
%  -options.regularisedGradientBased: Boolean variable to perform the
%   optimisation of the EFS dimensions. 
% Output variables
% *model_information: struct variable composed of: 
%  -projectedTrain: Projected train patterns (estimated latent variable 
%   for the train patterns)
%  -predictedTrain: Predicted target for train patterns
%  -projectedTest: Projected test patterns (estimated latent variable 
%   for the test patterns)
%  -model: struct variable composed of a projection vector and a set of 
%   thresholds
% *mae_error: MAE obtained (common error measure used in ordinal 
%  classification) for the selected algorithm and configuration
	  addpath tools

          if options.kernelized 
              % Construct kernel matrices
              if numel(param.kernelSigma) == 1,
                    kernelMatrixTrain = computeKernelMatrix(train.patterns, train.patterns, param.kernelSigma);
                    kernelMatrixTest = computeKernelMatrix(train.patterns, test.patterns, param.kernelSigma);
              else
                    kernelMatrixTrain = zeros(numel(train.targets),numel(train.targets));
                    kernelMatrixTest = zeros(numel(train.targets),numel(test.targets));
                    for i=1:numel(param.kernelWeight)
                          kernelMatrixTrain = kernelMatrixTrain + param.kernelWeight(i)*computeKernelMatrix(train.patterns, train.patterns, param.kernelSigma(i));
                          kernelMatrixTest = kernelMatrixTest + param.kernelWeight(i)*computeKernelMatrix(train.patterns, test.patterns, param.kernelSigma(i));
                    end
              end

              % Optimise the dimensions chosen for the EFS
              if options.regularisedGradientBased,
                  [newTrain newTest] = calculateEmpiricalFeatureSpaceOptimisingTheEFSDimensions(kernelMatrixTrain,kernelMatrixTest,train,options.ordinal,param.lambda);
              else
                  [newTrain newTest] = calculateEmpiricalFeatureSpace(kernelMatrixTrain,kernelMatrixTest,param.EFSdimensions);
              end
          else
              newTrain = train;
              newTest = test;
          end
          
          newTrain.targets = train.targets;
          newTest.targets = test.targets;
                
          % Train the model
          [model] = train_kPOM(newTrain);
          
          % Test the model
          [model_information.projectedTrain,model_information.predictedTrain] = test_kPOM(newTrain.patterns, model);
          [model_information.projectedTest,model_information.predictedTest] = test_kPOM(newTest.patterns, model);
          model_information.model = model;
          mae_error = mean(abs(model_information.predictedTrain - train.targets));
end
        
