%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) María Pérez Ortiz (i82perom at uco dot es)
% 
% This code implements the kernelised versions of the Proportional Odds
% Model presented in the paper: Kernelising the Proportional Odds Model
% through Kernel Learning techniques
% 
% The code has been tested with Ubuntu 12.04 x86_64 and Matlab R2009a
% 
% If you use this code, please cite the associated paper
% Code updates and citing information:
% http://www.uco.es/grupos/ayrna/neucom-kpom
% 
% AYRNA Research group's website:
% http://www.uco.es/ayrna 
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 3
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
% Licence available at: http://www.gnu.org/licenses/gpl-3.0.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [newTrain newTest] = calculateEmpiricalFeatureSpaceOptimisingTheEFSDimensions(kernelMatrixTrain,kernelMatrixTest,train,ordinal,lambda)
% Calculate the EFS associated to a kernel matrix and optimise the
% dimensions of the EFS via a regularised gradient ascent technique based
% on kernel-target alignment. 
% Input variables
%  *kernelMatrixTrain: train kernel matrix
%  *kernelMatrixTest: test kernel matrix
%  *train: train structure containing the train patterns and the associated
% targets
%  *ordinal: boolean variable to include the ordinal weights for the
% learning phase
%  *lambda: lambda parameter for the regularisation
% Output variables
%  *newTrain: train patterns in the reduced EFS
%  *newTest: test patterns in the reduced EFS
         
         % Choose a tolerance value for the eigenvalues
         toleranceEigen = max(size(kernelMatrixTrain))*eps(norm(kernelMatrixTrain));
         % Diagonalise the train kernel matrix
         [V,D] = eig(kernelMatrixTrain);
         a = sum(D);

         % Remove indices lower than the fixed tolerance value
         indexes = a>toleranceEigen;
         positiveEigenvalues = diag(a(indexes));
         eigenvectors = V(:,indexes);
         
         % Compute the ideal kernel matrix
         ideal_grammatrix = zeros(numel(train.targets),numel(train.targets));
         for i=1:size(train.patterns,1),
             ideal_grammatrix(i,:) = (train.targets == train.targets(i)); 
         end
         
         % Apply ordinal weights if necessary
         if ordinal
             weights = zeros(numel(train.targets),numel(train.targets));
             for i=1:size(train.patterns,1),
                 weights(i,:) = abs(train.targets - train.targets(i)); 
             end
             weights(weights==0) = 1;
             ideal_grammatrix = ideal_grammatrix .* weights;
         end
         
         % Initialise weights
         initials = rand(1,sum(indexes));
         % Configure the gradient ascent method             
         f = @(x,autoval,autovec,ideal_grammatrix,lambda)alignmentLMIKDerivative(x,autoval,autovec,ideal_grammatrix,lambda);
         parm.method = 'IRprop+';
         parm.d_Obj = -1;
         parm.TolX = 0;
         parm.full_hist = true;
         parm.MaxIter = 100;
         [x,e,exitflag,stats] = rprop(f,initials,parm,positiveEigenvalues,eigenvectors,centerkernel(ideal_grammatrix),lambda);
                        
         [m,index] = min(stats.error);
         solutions = sigmoid(stats.x{index});

         % We save the indices which weight obtained is lower than 10^-8
         indexes = (solutions>10^-8);
         if sum(indexes) == 0,
              disp('All the indixes were discarded. Choosing the 10 indexes associated to greatest weights')
              [y,ind] = sort(solutions'.* diag(positiveEigenvalues),1,'descend');
              indexes = zeros(1,numel(ind));
              indexes(ind(1:10)) = 1;
         end
         
         % Recompose the new eigenvalue matrix
         positiveEigenvalues = positiveEigenvalues(indexes==1,indexes==1);
         % The same for the eigenvectors
         eigenvectors = eigenvectors(:,indexes==1);
                     
         product = diag(1./(sqrt(sum(positiveEigenvalues))))*eigenvectors';
         % Project the patterns
         newTrain.patterns = kernelMatrixTrain'*product';
         newTest.patterns = kernelMatrixTest'*product';
                    
end
