%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) María Pérez Ortiz (i82perom at uco dot es)
% 
% This code implements the kernelised versions of the Proportional Odds
% Model presented in the paper: Kernelising the Proportional Odds Model
% through Kernel Learning techniques
% 
% The code has been tested with Ubuntu 12.04 x86_64 and Matlab R2009a
% 
% If you use this code, please cite the associated paper
% Code updates and citing information:
% http://www.uco.es/grupos/ayrna/neucom-kpom
% 
% AYRNA Research group's website:
% http://www.uco.es/ayrna 
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 3
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
% Licence available at: http://www.gnu.org/licenses/gpl-3.0.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function sol = optimiseCombinationOfKernels(train,parameters,ordinal)
% Optimise a linear combination of kernel matrices via kernel-target
% alignment
% Input variables
% *train: train structure containing the train patterns and the associated
% targets
% *model: kernel width parameters associated to the kernels that are to be
% combined
% *ordinal: boolean variable to include the ordinal weights for the
% learning phase
% Output variables
% *sol: weight parameter for each kernel

            % Default parameters to try
            if nargin < 2,
                parameters = [10^-3, 10^-2, 10^-1, 10^0, 10^1, 10^2, 10^3];
            end
            
            % Compute the ideal kernel matrix
            a = zeros(numel(parameters),1);
            ideal_grammatrix = zeros(numel(train.targets),numel(train.targets));
            for i=1:size(train.patterns,1),
            	ideal_grammatrix(i,:) = (train.targets == train.targets(i)); 
            end
            
            % Apply ordinal weights if necessary
            if ordinal
                 weights = zeros(numel(train.targets),numel(train.targets));
                 for i=1:size(train.patterns,1),
                     weights(i,:) = abs(train.targets - train.targets(i)); 
                 end
                 weights(weights==0) = 1;
                 ideal_grammatrix = ideal_grammatrix .* weights;
            end

            % Matrices to be combined
            kernelMatrixes = zeros(numel(train.targets),numel(train.targets)*numel(parameters));

            M = zeros(numel(parameters),numel(parameters));

            % Compute all the necessary information for the learning phase
            for i=1:numel(parameters),
                    k = computeKernelMatrix(train.patterns, train.patterns,parameters(i));
                    kernelMatrixes(:,(1+(i-1)*numel(train.targets)):(i*numel(train.targets))) = (centerkernel(k));%/trace(centerkernel(k));
                    a(i) = frobeniusInnerProduct(kernelMatrixes(:,(1+(i-1)*numel(train.targets)):(i*numel(train.targets))),ideal_grammatrix);
            end

            for i=1:numel(parameters),
                for j=1:numel(parameters),
                        M(i,j) = frobeniusInnerProduct(kernelMatrixes(:,(1+(i-1)*numel(train.targets)):(i*numel(train.targets))),  kernelMatrixes(:,(1+(j-1)*numel(train.targets)):(j*numel(train.targets))));
                end
            end
            
            % Learn a combination of the computed kernels
            [x] = quadprog(M,-2*a,[],[],[],[],zeros(1,numel(parameters)),ones(1,numel(parameters))*inf,ones(1,numel(parameters))*(1/numel(parameters)));
            % Normalize the solution
            sol = x/norm(x,1);
                
end
